require "import"
import "android.content.Context"
import "android.os.Vibrator"
import "android.media.ToneGenerator"
import "android.media.AudioManager"
import "android.widget.*"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "android.text.Html"
import "android.content.ClipData"
import "android.content.ClipboardManager"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(40)

-- إضافة جملة الإدخال هنا
local translationResult = ""

local languageList = {
    {"الكانادا", "kn"},
    {"التاميل", "ta"},
    {"المالايالامية", "ml"},
    {"الإنجليزية", "en"},
    {"الأفريكانية", "af"},
    {"الألبانية", "sq"},
    {"الأمهرية", "am"},
    {"العربية", "ar"},
    {"الأرمنية", "hy"},
    {"الأذربيجانية", "az"},
    {"الباسكية", "eu"},
    {"البيلاروسية", "be"},
    {"البنغالية", "bn"},
    {"البوسنية", "bs"},
    {"البلغارية", "bg"},
    {"الكاتالانية", "ca"},
    {"السيبيونية", "ceb"},
    {"الشيتشيوة", "ny"},
    {"الصينية (المبسطة)", "zh-CN"},
    {"الصينية (التقليدية)", "zh-TW"},
    {"الكورسيكية", "co"},
    {"الكرواتية", "hr"},
    {"التشيكية", "cs"},
    {"الدانماركية", "da"},
    {"الهولندية", "nl"},
    {"الإسبرانتو", "eo"},
    {"الإستونية", "et"},
    {"الفلبينية", "tl"},
    {"الفنلندية", "fi"},
    {"الفرنسية", "fr"},
    {"الفريزية", "fy"},
    {"الجاليكية", "gl"},
    {"الجورجية", "ka"},
    {"الألمانية", "de"},
    {"اليونانية", "el"},
    {"الجوجاراتية", "gu"},
    {"الهايتية الكريولية", "ht"},
    {"الهوسا", "ha"},
    {"الهوايان", "haw"},
    {"العبرية", "he"},
    {"الهندية", "hi"},
    {"الهامونج", "hmn"},
    {"الهنغارية", "hu"},
    {"الأيسلندية", "is"},
    {"الإيجبو", "ig"},
    {"الإندونيسية", "id"},
    {"الأيرلندية", "ga"},
    {"الإيطالية", "it"},
    {"اليابانية", "ja"},
    {"الجاوية", "jv"},
    {"الكازاخستانية", "kk"},
    {"الخميرية", "km"},
    {"الكورية", "ko"},
    {"الكردية", "ku"},
    {"القيرغيزية", "ky"},
    {"اللاوية", "lo"},
    {"اللاتينية", "la"},
    {"اللاتفية", "lv"},
    {"اللتوانية", "lt"},
    {"اللوكسمبورغية", "lb"},
    {"المقدونية", "mk"},
    {"المالاغاسية", "mg"},
    {"الماليزية", "ms"},
    {"المالطية", "mt"},
    {"الماورية", "mi"},
    {"الماراثية", "mr"},
    {"المنغولية", "mn"},
    {"الماينمارية (البورمية)", "my"},
    {"النيبالية", "ne"},
    {"النرويجية", "no"},
    {"الباشتو", "ps"},
    {"الفارسية", "fa"},
    {"البولندية", "pl"},
    {"البرتغالية", "pt"},
    {"البنجابية", "pa"},
    {"الرومانية", "ro"},
    {"الروسية", "ru"},
    {"الساموية", "sm"},
    {"الغالية الأسكتلندية", "gd"},
    {"الصربية", "sr"},
    {"السوتو", "st"},
    {"الشونا", "sn"},
    {"السندية", "sd"},
    {"السنهالية", "si"},
    {"السلوفاكية", "sk"},
    {"السلوفينية", "sl"},
    {"الصومالية", "so"},
    {"الإسبانية", "es"},
    {"السوندانية", "su"},
    {"السواحيلية", "sw"},
    {"السويدية", "sv"},
    {"الطاجيكية", "tg"},
    {"التيلوجو", "te"},
    {"التايلاندية", "th"},
    {"التركية", "tr"},
    {"الأوكرانية", "uk"},
    {"الأردية", "ur"},
    {"الأوزبكية", "uz"},
    {"الفيتنامية", "vi"},
    {"الولزية", "cy"},
    {"الخوسية", "xh"},
    {"اليديشية", "yi"},
    {"اليوروبية", "yo"},
    {"الزولو", "zu"}
}

local items = {}
for _, languageInfo in ipairs(languageList) do
    table.insert(items, languageInfo[1]) -- إضافة أسماء اللغات إلى قائمة العناصر
end

function capitalizeFirstWord(str)
    return str:gsub("^%l", string.upper)
end

function addPunctuation(str)
    return str .. "."
end

function hasInternetConnection()
    local connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

if not hasInternetConnection() then
    service.asyncSpeak("لا يوجد اتصال بالإنترنت. لا يمكن ترجمة اللغة")
    return false
end

local layout = {
    LinearLayout;
    layout_width = "fill";
    layout_height = "fill";
    orientation = "vertical";
    {
        LinearLayout;
        layout_width = "fill";
        layout_height = "wrap";
        orientation = "horizontal";
        gravity = "center_vertical";
        {
            TextView;
            layout_width = "0dp";
            layout_height = "wrap";
            layout_weight = 1;
            gravity = "center";
            text = "ترجمة جوجل";
        };
        {
            Button,
            id = "telegramButton";
            layout_width = "wrap";
            layout_height = "wrap";
            text = "انضم إلى قناتنا على التليجرام";
        };
    };
    {
        EditText,
        id = "searchBox";
        layout_width = "fill";
        layout_height = "wrap";
        hint = "ابحث عن اللغة التي تريد الترجمة إليها...";
    };
    {
        GridView,
        id = "grid";
        numColumns = 1;
        layout_width = "fill";
        layout_height = "0dp";
        layout_weight = 1;
    },
    {
        Button,
        id = "exitButton";
        layout_width = "fill";
        layout_height = "wrap";
        text = "خروج";
    }
}

local dlg = LuaDialog(context)
dlg.View = loadlayout(layout)

local adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, items)
grid.adapter = adapter

dlg.setTitle("")
    .show()

searchBox.addTextChangedListener{
    onTextChanged = function(s, start, before, count)
        local keyword = s.toString()
        local filteredItems = {}
        for _, language in ipairs(items) do
            if language:lower():find(keyword:lower()) then
                table.insert(filteredItems, language)
            end
        end
        adapter.clear()
        adapter.addAll(filteredItems)
        adapter.notifyDataSetChanged()
    end
}
grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(500, function()
        local language = v.text
        local defaultLanguage = Locale.getDefault().getLanguage()
        local languageCode
        for _, languageInfo in ipairs(languageList) do
            if languageInfo[1] == language then
                languageCode = languageInfo[2]
                break
            end
        end

        if languageCode then
            local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(context.getText(node))
            url = url .. "&tl=" .. languageCode

            Http.get(url, function(status, result)
                if status == 200 then
                    local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
                    translation = capitalizeFirstWord(translation)
                    translation = addPunctuation(translation)
                    translationResult = translationResult .. translation
                    
                    -- إنشاء وعرض مربع حوار الترجمة
                    local translationDialog = LuaDialog(context)
                    translationDialog.setTitle("الترجمة")
                        .setMessage(translation)
                        .setPositiveButton("موافق", nil)
                        .setNeutralButton("نسخ", function()
                            local clipboardManager = context.getSystemService(Context.CLIPBOARD_SERVICE)
                            local clip = ClipData.newPlainText("النص المترجم", translation)
                            clipboardManager.setPrimaryclipboardManager.setPrimaryClip(clip)
                            Toast.makeText(context, "تم النسخ إلى الحافظة", Toast.LENGTH_SHORT).show()
                            service.copy(translation)
                            -- مسح محتوى الترجمة بعد النسخ
                            translationResult = ""
                        end)
                        .show()
                        
                    service.speak("ترجمة النص: " .. translation)
                    vibrator.vibrate(100)
                else
                    print("عذرًا، حدث خطأ.\n قد تكون هذه الطرق مفيدة:\n - تأكد من أنك متصل بالإنترنت.\n - حاول توفير اتصال إنترنت مستقر.\n ملاحظة: بعض العناصر، الصور، صناديق النصوص، إلخ، قد لا تُترجم.")
                    vibrator.vibrate(200)
                    service.playSoundTick()
                end
            end)
        else
            print("اللغة غير مدعومة")
            vibrator.vibrate(200)
            service.playSoundClock()
        end
    end)
end

telegramButton.onClick = function(view)
    dlg.dismiss()
    local url = "https://t.me/AndroidTechnologyForTheBlind"
    local intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
    context.startActivity(intent)
end

exitButton.onClick = function(view)
    service.playSoundTick()
    dlg.dismiss()
end